require "English"
require "cmd_line.rb"

# bif2jb_bif.rb -i* C:\data\uni\bn\original_bn_rep -> implicitly output into that dir as well.

def quote(x)
	x = "\"#{x}\"" unless x =~ /".*"/
	return x
end

def putIntoArray(filesWithProbs, probs)
	probs = probs.split(" ").map!{|x|x.to_f}
#	p probs
	filesWithProbs[probs.length] = [] unless filesWithProbs[probs.length]
	filesWithProbs[probs.length] << probs
	filesWithProbs
end

# #########################
# MAIN: Convert specified input_files from .bif format to .jb_bif format.
# #########################
input_files = read_in("\\.bif$")
filesWithBNProbs = []
for infile_name in input_files
	puts infile_name 
	#==== Open input and output file.
	outfile_name = infile_name.sub(/\.bif$/, ".jb_bif")
	puts "  Converting #{infile_name} -> #{outfile_name}"
	infile = File.open(infile_name.gsub(/\//,"\\"))
	outfile = File.open(outfile_name, "w") 

	#==== Read complete input file.
	content = infile.gets(nil) 

	#==== Get rid of all comments.
	content.gsub!(/\/\/.*?\n/, " ") # ? => not greedy !!
	content.gsub!(/\n/, " ") 
	
	#==== Process header.
	content =~ /network\s+([^\s\{]*)/
	name, content = $1, $POSTMATCH
	outfile.puts "network #{quote(name)} {\n}"

	# #########################
	# PROCESS VARIABLES.
	# #########################

	while content =~ /variable\s+([^\s\{]*)(.*?\})/ #only read to first } in varblock
		varname, varblock, content = $1, $2, $POSTMATCH
		outfile.puts("variable #{quote(varname)}{")
		while varblock =~ /[^\{]*property\s+([^;]+);/
			property, varblock = $1, $POSTMATCH
			outfile.puts("property #{quote(property)};")
		end	
			
		
		#==== Process varblock.
		varblock =~ /type discrete\s*\[\s*(\d+)[^\{]+\{(.*)\}/ # get domain_size and value names 
		num_dim, dims = $1, $2
		outfile.puts("  type discrete[#{num_dim}] {")
		values = dims.split( " " )
		values.map!{|x| x.sub(/,/,"")} # delete commas
		values.each{|x| outfile.puts("    \"#{x}\"")}
		
		outfile.puts("  };")

		while varblock =~ /property([^;]+);/
			property, varblock = $1, $POST_MATCH
			outfile.puts("property #{quote{property}};")
		end	

		outfile.puts("}")
	end

	# #########################
	# PROCESS PROBABILITIES.
	# #########################
	while content =~ /probability\s*\(([^\)]*)\)\s*\{([^\}]*)\}/ # match parts between () and between {}
		vars, probrest, probblock, content = $1, $2, $MATCH, $POSTMATCH
		
		#===== Handle probability header.
		vars.gsub!(/,|\|/,"")
		vars = vars.split( " " )
		outstring = "probability ( "
		vars.each{|x| outstring = outstring + "#{quote(x)} " }
		outfile.puts outstring + ") {"
		
		# Handle one property of the probability.
		if probrest =~ /^[^\(]*property ([^;]*);/
			outfile.puts "property \"" + $1 + "\" ;"
		end

		#===== Handle probability table.
		probrest.gsub!(/,/, "") # Erase commas.
		if probrest =~ /^\s*table/ # table. Easy case.
			outfile.puts(probrest)
			filesWithBNProbs = putIntoArray(filesWithBNProbs, probrest.sub(/^\s*table/,""))
		else
			while probrest =~ /\(([^\)]*)\)([^;]*);/ # match part in (), each line ends with ;
				values, table, probrest = $1, $2, $POSTMATCH
				values = values.split( " " )
				outstring = "  ( "
				values.each{|x| outstring = outstring + "#{quote(x)} " }
				outfile.puts outstring + ")#{table} ;"
				filesWithBNProbs = putIntoArray(filesWithBNProbs, table)
			end
		end
		outfile.puts "}"
	end
	#==== Tidy up.
	infile.close
	outfile.close
end
for i in 1..filesWithBNProbs.length
	next unless filesWithBNProbs[i]
	file = File.open(file_with_bn_probs(i), "w")
	for probs in filesWithBNProbs[i]
		file.puts probs.join(" ")
	end
	file.close
end