require "English"
#!/bin/env ruby

class Array
	def rest
		return self[1..-1] # first to last
	end
end

def windows()
#	p "platform", PLATFORM
	return PLATFORM.downcase =~ /win/ 
end



def default_sort(x,y)
	if x =~ /(.*?)-*(#{float_regexp})/
		valname, val, rest_x = $1, $2.to_f, $POSTMATCH
		if y =~ /(.*?)-*(#{float_regexp})/
			valname2, val2, rest_y = $1, $2.to_f, $POSTMATCH
			if valname == valname2
				return default_sort(rest_x, rest_y) if val == val2
				return -1 if val < val2
				return 1 
			end
			return valname<=>valname2
		end
	end
	raise "#{x} and #{y} are floats, but are not compared correctly with default_sort" if x =~ /#{float_regexp}/ and y =~ /#{float_regexp}/
	return x<=>y
end

def gv()
	windows ? "gsview32" : "gv"
end

def path()
	return ""
#	return "/guest1/hutter/testbed/" if PLATFORM == "i386-linux-gnu"
#	windows ? "C:/testbed/" : "~/testbed/"#"/u/kiew/student/hutter/testbed/"
end

def file_with_bn_probs( length )
	path + "/bn_probs#{length}.txt"
end

def sls_executable()
	windows ? path + "solvers\\fh\\Debug\\sls.exe" : path + "solvers/sls"
end

def bbmb_executable()
	path + "solvers/bbmb"
end

def inststats_filename()
	return path + "results/instance-stats.hash"
end

def fdc_filename()
	return path + "results/fdc.hash"
end

def runstats_filename(algo)
	return path + "runstats-" + algo.sub(/-runstats/,"") + ".hash"
end

def instance_runstats_filename(inst)
	return path + "instance_results/" + inst + "-run-stats.hash"
end

def runtime_filename()
	return path + "results/runtime.hash"
end

def optsol_filename()
	return path + "results/opt-sols.hash"
end

def myoptsol_filename()
	return path + "results/my-opt-sols.hash"
end

def mdstat_filename()
	return path + "results/md-stat.hash"
end

def gnuplot()
	windows ? "pgnuplot" : "gnuplot"
end

def need_help()
	return ( $ARGV.length < 1 or $ARGV[0] =~ /^-(-)?h(elp)?$/ )
end

def call(command, *indentation)
	indentation[0].times{|x| print " "} if indentation[0]
	puts "Calling: " + command
	system command
end

def float_regexp()
	return '[+-]?\d+(?:\.\d+)?(?:[eE][+-]\d+)?';
end

def get_final_value(filename)
	time_step_it = qual = std = 0
	File.open(filename){|file|
		while line=file.gets
			save_line=line 
		end
		time_step_it,qual,std = save_line.split.map{|x|x.to_f}
	}
	return [time_step_it, qual, std] 
end

def get_time_steps_its_qual_std(sls_filename)
	filename = sls_filename.sub(/\.sls$/,".gpt")
	call "ruby -S sls2data.rb -i #{sls_filename} -no_out" unless File.exist?(filename)
	
	time, qual, std = get_final_value(filename)
	steps,qual,std = get_final_value(sls_filename.sub(/\.sls$/,".gpf"))
	its,qual,std = get_final_value(sls_filename.sub(/\.sls$/,".gpi"))
	return [time, steps, its, qual, std]
end

# #########################
# Return true iff indicator appears in $ARGV.
# #########################
def check_arg(indicator)
	return $ARGV.index(indicator)
#	return (i != nil)
end

# #########################
# Read smth. from $ARGV which follows the indicator
# nil if indicator doesn't appead in $ARGV
# error if indicator is the last element of $ARGV.
# #########################
def read_smth(indicator)
	i=0
	result = nil
	if (i=$ARGV.index(indicator))
		result = $ARGV[i+1] 
	end
	return result
end

# #########################
# Return an array with all the arguments after the indicator.
# #########################
def read_all_after(indicator)
	if i=$ARGV.index(indicator)
		k=i+1
		k=k+1 while k < $ARGV.length and not $ARGV[k] =~ /^-/
		result = $ARGV[i+1...k]
		return result
	else
		puts "Must have #{indicator} in the command line.\nExiting."
		exit -1
	end
end

# #########################
# Read smth. from $ARGV which follows 2 args after the indicator
# nil if indicator doesn't appead in $ARGV
# error if indicator is the last element of $ARGV.
# #########################
def read_smth_2after(indicator)
	i=0
	result = nil
	if (i=$ARGV.index(indicator))
		result = $ARGV[i+2] 
	end
	return result
end

# #########################
# Read an integer from $ARGV which follows the indicator. Use default value if not an argument and default specified. 
# #########################
def read_int(indicator, *default)
	if result = read_smth(indicator)
		return result.to_i if result =~ /^\d+$/
		puts "Argument #{result} following #{indicator} is supposed to be an integer.\nExiting."
		exit -1
	end
	unless default
		puts "Must specify integer argument following #{indicator}\nExiting." 
		exit -1
	end
	return default[0]
end

# #########################
# Read a float from $ARGV which follows the indicator. Use default value if not an argument and default specified. 
# #########################
def read_float(indicator, *default)
	if result = read_smth(indicator)
		return result.to_f if result =~ /#{float_regexp}/
		puts "Argument #{result} following #{indicator} is supposed to be a float.\nExiting."
		exit -1
	end
	unless default
		puts "Must specify float argument following #{indicator}\nExiting." 
		exit -1
	end
	return default[0]
end

# #########################
# Read a file name from $ARGV which follows the indicator and assure that it exists.
# #########################
def read_filename(indicator)
	unless result = read_smth(indicator)
		puts "You must specifiy a filename after #{indicator}.\nExiting."
		exit(-1)
	end
	unless FileTest.exist?(result) 
		puts "Specified file #{result} does not exist.\nExiting."
		exit(-1)
	end
	return result
end

# #########################
# Read a dir name from $ARGV which follows the indicator, 
# assure that it exists and put a separator to its end if there is none yet.
# #########################
def read_dirname(indicator)
	result = read_filename(indicator)
#	result = File.expand_path(result)
	result = result + File::Separator unless( result =~ /[\/\\]$/ )
	return result
end

# #########################
# Read all consecutive integers from $ARGV starting at indicator.
# #########################
def read_integers_def(indicator, default)
	if i=$ARGV.index(indicator)
		result = []
		j=i+1
		while j<$ARGV.length and $ARGV[j] =~ /^\d+$/ do
			result << $ARGV[j].to_i 
			j=j+1
		end
		return result
	else return default end
end

# #########################
# Flexible reading of input filenames.
# #########################
def read_in(*additional_regexps)
	result = []
	if check_arg("-i") # just one file
		result << read_filename("-i")
	elsif check_arg("-i*") # all files in a directory
		dir = read_dirname("-i*")
		p dir
		Dir.foreach(dir){|x|
			next if x =~ /^(\.|\.\.)$/ # skip . and ..
			result << dir + "" + x
		}
	elsif check_arg("-dr") # all files in a directory matching a regexp
		dir = read_dirname("-dr")
		regexp = read_smth_2after("-dr")
		Dir.foreach(dir) {|x| 
			next if x =~ /^(\.|\.\.)$/ # skip . and ..
			next unless x =~ /#{regexp}/
			result << dir +"" + x if x =~ /#{regexp}/ 
		}
	elsif check_arg("-l") # explicit list of files
		i=$ARGV.index("-l")
		result = $ARGV[i+1..$ARGV.length-1]
	else
		puts "Need to specify input file(s). Options:"
		puts "-i  followed by one file"
		puts "-i* followed by a directory out of which every file is taken"
		puts "-dr followed by a directory dir and a regular expression reg. All files in dir matching reg are taken."
		puts "-l  (as the last parameter) followed by a list of input files."
		puts 'Example: <ruby script> -dr ../instances/ \.bif$  (all files in directory ../instances/ ending with .bif)'
		puts "======="
		puts "Exiting."
		exit(-1)
	end

	#=== Remove elements that don't match the additional regexps
	for exp in additional_regexps
		result.delete_if{|x| not x =~ /#{exp}/}
	end
	
	#=== Return full file names
#	result.map!{|x| File.expand_path(x)}
	return result
end