package mpe;
import java.util.Hashtable;

/**
 * Creation date: (20.01.2004 17:53:32)
 * @author Frank Hutter
 */
public class MaxCSPInstance {
	String name;
	
	int numVars;
	int numConstraints;
	CSPVariable[] variables;
	Constraint[] constraints; 
	
	/**
	 * 
	 */
	public MaxCSPInstance(BayesianNetwork bn) {
		name = bn.name;
		numVars = bn.numVars;
		variables = new CSPVariable[numVars];
		
		Hashtable original_index_2_csp_index = new Hashtable();
		//=== Construct CSPVars and determine number of constraints.
		numConstraints = 0;
		double summedWeight = 0;
		for (int i = 0; i < bn.numVars; i++) {
			BNVariable var = bn.variables[i];
			variables[i] = new CSPVariable(var);
			original_index_2_csp_index.put(new Integer(variables[i].index), new Integer(i));
			numConstraints += var.CPT.length;
			for (int j = 0; j < var.CPT.length; j++) {
				if(var.CPT[j] != 0) summedWeight += Math.log(var.CPT[j]);
			}
		}
		
		//=== Construct constraints.
		constraints = new Constraint[numConstraints];
		
		int numConstructedConstraints = 0;
		for (int i = 0; i < bn.numVars; i++) {
			BNVariable var = bn.variables[i];
			
			//=== Construct constraint variables for var.
			CSPVariable[] constraintVars = new CSPVariable[var.numParents+1];
			constraintVars[0] = variables[i];
			for (int j = 0; j < var.numParents; j++) {
				int csp_index = ((Integer) original_index_2_csp_index.get(new Integer(var.parents[j].index))).intValue();
				constraintVars[j+1] = variables[csp_index];
			}			
			
			//=== Construct constraints for variable i.
			for (int j = 0; j < var.CPT.length; j++) {
				double weight = 0;
				if( var.CPT[j] == 0 ) weight = summedWeight - 1; //as summedWeight < 0
				else weight = Math.log(var.CPT[j]) / Math.log(10); // log10(x) = ln(x) / ln(10)
				constraints[numConstructedConstraints++] = new Constraint(constraintVars, j, weight);
			}
		}
	}
	
	public String toString(){
		StringBuffer toResult = new StringBuffer();
		toResult.append("MAX CSP Instance\n");
		toResult.append("================\n");
		toResult.append("Origin: BN " + name + "\n\n");
		toResult.append("Variables:\n");
		for (int i = 0; i < variables.length; i++) {
			toResult.append(variables[i] +"\n");
		}
		toResult.append("\nConstraints:\n");
		for (int i = 0; i < constraints.length; i++) {
			toResult.append(constraints[i] + "\n");
		}
		return toResult.toString();
	}
}
