package mpe;

import java.util.Hashtable;
import java.util.Vector;

/**
 * Creation date: (20.01.2004 20:00:25)
 * @author Frank Hutter
 */
public class MaxSATInstance {
	String name;
	int numVars;
	int numClauses;
	SATVariable[] vars;
	Clause[] clauses;
	MaxCSPInstance WCSP;
	Hashtable csp_var_2_var_vector;
	
	public MaxSATInstance(MaxCSPInstance newWCSP){
		WCSP = newWCSP;
		name = WCSP.name;
		csp_var_2_var_vector = new Hashtable();
		double summedWeight = 0;
		
		numVars = 0;
		int maxNumClauses = WCSP.numConstraints + WCSP.numVars; // one for each var stating that it has some value. 
		for (int i = 0; i < WCSP.numVars; i++) {
			CSPVariable var = WCSP.variables[i];
			numVars += var.domainSize;
			//numClauses += (var.domainSize *(var.domainSize-1)/2); // one for each pair that they're not both true.
		}
		vars = new SATVariable[numVars+1];
		Clause[] to_clauses = new Clause[maxNumClauses];

		int varsSoFar = 1;
		for (int i = 0; i < WCSP.numVars; i++) {
//			System.out.println("Converting CSP variable " + (i+1) + " of " + WCSP.numVars);
			CSPVariable var = WCSP.variables[i];
			for (int j = 0; j < var.domainSize; j++) {
				vars[varsSoFar] = new SATVariable(varsSoFar,i,j, var.name);

				//=== Keep track of the SAT variable referring to the CSP variable
				Vector v;
				if( csp_var_2_var_vector.containsKey(var) ){
					v = (Vector) csp_var_2_var_vector.get(var);
				} else {
					v = new Vector();
				}
				v.add(vars[varsSoFar]);
				csp_var_2_var_vector.put(var, v);
				
				varsSoFar++;
			}
		}
		
		//=== Construct clauses.
		int clausesSoFar = 0;
		
		//=== First, construct one clause for each constraint.
		for (int i = 0; i < WCSP.numConstraints; i++) {
//			System.out.println("Converting constraint " + (i+1) + " of " + WCSP.numConstraints);
			Constraint cons = WCSP.constraints[i];
			
			//=== Determine variables that have to be in the clause.
			SATVariable[] clauseVars = new SATVariable[cons.vars.length];
			for (int j = 0; j < cons.vars.length; j++) {
				Vector vSatVars = (Vector) csp_var_2_var_vector.get(cons.vars[j]);
				SATVariable correctSatVar = null;
				for(int k = 0; k < vSatVars.size(); k++) {
					correctSatVar = (SATVariable) vSatVars.elementAt(k);
					if(correctSatVar.csp_value == cons.values[j]) break;
				}
				clauseVars[j] = correctSatVar;
			}
			
			//=== Construct the clause.
			boolean[] truthVals = new boolean[clauseVars.length];
			for (int j = 0; j < truthVals.length; j++) {
				truthVals[j] = false;
			}
			double weight = cons.weight;
			if ( weight == 0 ){
				continue;
			}
			else weight = -weight;
			
			to_clauses[clausesSoFar++] = new Clause(clauseVars, truthVals, weight);
			summedWeight += weight;
		}
				
		
/*		//=== Now, construct one clause for each pair of SAT variables corresponding to the 
		//=== same CSP variables, stating that at least one of them must be false.
		for (int i = 0; i < WCSP.numVars; i++) {
			Vector v = (Vector) csp_var_2_var_vector.get(WCSP.variables[i]);
			SATVariable[] satVars = new SATVariable[v.size()]; 
			for (int j = 0; j < v.size(); j++) {
				satVars[j] = (SATVariable) v.elementAt(j);
			}
			boolean[] truthVals = new boolean[2];
			truthVals[0] = false;
			truthVals[1] = false;
			for (int j = 0; j < satVars.length; j++) {
				SATVariable[] clauseVars = new SATVariable[2];
				clauseVars[0] = satVars[j]; 
				for (int k = j+1; k < satVars.length; k++) {
					clauseVars[1] = satVars[k];
					clauses[clausesSoFar++] = new Clause(clauseVars, truthVals, 100000);
				}
			}
		}
	*/
		//=== Now, construct constraints for each var in the Max-CSP problem
		//=== stating that this var must take at least one value.
		for (int i = 0; i < WCSP.numVars; i++) {
//			System.out.println("Now constructing constraint stating that var " + (i+1) 
//			         + "/" + WCSP.numVars + " must have at least one value." );
			Vector v = (Vector) csp_var_2_var_vector.get(WCSP.variables[i]);
			SATVariable[] clauseVars = new SATVariable[v.size()]; 
			for (int j = 0; j < v.size(); j++) {
				clauseVars[j] = (SATVariable) v.elementAt(j);
			}
			boolean[] truthVals = new boolean[clauseVars.length];
			for (int j = 0; j < truthVals.length; j++) {
				truthVals[j] = true;
			}
			to_clauses[clausesSoFar++] = new Clause(clauseVars, truthVals, summedWeight+1);
		}
		numClauses = clausesSoFar;
		clauses = new Clause[numClauses];
		for (int i = 0; i < numClauses; i++) {
			clauses[i] = to_clauses[i];
		}
	}
	
	public String toString(){
//		System.out.println("Returning string representation of MaxSATInstances");
		StringBuffer toResult = new StringBuffer();
		toResult.append("c Weighted MAX-SAT Instance\n");
		toResult.append("c =========================\n");
		toResult.append("c Origin: Bayesian Network " + name + "\n");
		for (int i = 1; i <= numVars; i++) {
			toResult.append("c " + vars[i].index + " <=> " + vars[i].csp_name + "=" + vars[i].csp_value + " (" + vars[i].csp_var_index + "=" + vars[i].csp_value + ")\n"); 
		}
		toResult.append("p wcnf " + numVars + " " + clauses.length + "\n"); 
		for (int i = 0; i < clauses.length; i++) {
			toResult.append(clauses[i] + " 0\n");
		}
		return toResult.toString();
	}
}
